# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .buckets import (
    BucketsResource,
    AsyncBucketsResource,
    BucketsResourceWithRawResponse,
    AsyncBucketsResourceWithRawResponse,
    BucketsResourceWithStreamingResponse,
    AsyncBucketsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.ai import (
    embedding_url_params,
    embedding_list_params,
    embedding_create_params,
    embedding_similarity_search_params,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.embedding_response import EmbeddingResponse
from ....types.ai.embedding_list_response import EmbeddingListResponse
from ....types.ai.embedding_retrieve_response import EmbeddingRetrieveResponse
from ....types.ai.embedding_similarity_search_response import EmbeddingSimilaritySearchResponse

__all__ = ["EmbeddingsResource", "AsyncEmbeddingsResource"]


class EmbeddingsResource(SyncAPIResource):
    @cached_property
    def buckets(self) -> BucketsResource:
        return BucketsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return EmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return EmbeddingsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        bucket_name: str,
        document_chunk_overlap_size: int | Omit = omit,
        document_chunk_size: int | Omit = omit,
        embedding_model: Literal["thenlper/gte-large", "intfloat/multilingual-e5-large"] | Omit = omit,
        loader: Literal["default", "intercom"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingResponse:
        """Perform embedding on a Telnyx Storage Bucket using the a embedding model.

        The
        current supported file types are:

        - PDF
        - HTML
        - txt/unstructured text files
        - json
        - csv
        - audio / video (mp3, mp4, mpeg, mpga, m4a, wav, or webm ) - Max of 100mb file
          size.

        Any files not matching the above types will be attempted to be embedded as
        unstructured text.

        This process can be slow, so it runs in the background and the user can check
        the status of the task using the endpoint `/ai/embeddings/{task_id}`.

        **Important Note**: When you update documents in a Telnyx Storage bucket, their
        associated embeddings are automatically kept up to date. If you add or update a
        file, it is automatically embedded. If you delete a file, the embeddings are
        deleted for that particular file.

        You can also specify a custom `loader` param. Currently the only supported
        loader value is `intercom` which loads Intercom article jsons as specified by
        [the Intercom article API](https://developers.intercom.com/docs/references/rest-api/api.intercom.io/Articles/article/)
        This loader will split each article into paragraphs and save additional
        parameters relevant to Intercom docs, such as `article_url` and `heading`. These
        values will be returned by the `/v2/ai/embeddings/similarity-search` endpoint in
        the `loader_metadata` field.

        Args:
          embedding_model: Supported models to vectorize and embed documents.

          loader: Supported types of custom document loaders for embeddings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/embeddings",
            body=maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "document_chunk_overlap_size": document_chunk_overlap_size,
                    "document_chunk_size": document_chunk_size,
                    "embedding_model": embedding_model,
                    "loader": loader,
                },
                embedding_create_params.EmbeddingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )

    def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingRetrieveResponse:
        """Check the status of a current embedding task.

        Will be one of the following:

        - `queued` - Task is waiting to be picked up by a worker
        - `processing` - The embedding task is running
        - `success` - Task completed successfully and the bucket is embedded
        - `failure` - Task failed and no files were embedded successfully
        - `partial_success` - Some files were embedded successfully, but at least one
          failed

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return self._get(
            f"/ai/embeddings/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingRetrieveResponse,
        )

    def list(
        self,
        *,
        status: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingListResponse:
        """
        Retrieve tasks for the user that are either `queued`, `processing`, `failed`,
        `success` or `partial_success` based on the query string. Defaults to `queued`
        and `processing`.

        Args:
          status: List of task statuses i.e. `status=queued&status=processing`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/ai/embeddings",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"status": status}, embedding_list_params.EmbeddingListParams),
            ),
            cast_to=EmbeddingListResponse,
        )

    def similarity_search(
        self,
        *,
        bucket_name: str,
        query: str,
        num_of_docs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingSimilaritySearchResponse:
        """
        Perform a similarity search on a Telnyx Storage Bucket, returning the most
        similar `num_docs` document chunks to the query.

        Currently the only available distance metric is cosine similarity which will
        return a `distance` between 0 and 1. The lower the distance, the more similar
        the returned document chunks are to the query. A `certainty` will also be
        returned, which is a value between 0 and 1 where the higher the certainty, the
        more similar the document. You can read more about Weaviate distance metrics
        here:
        [Weaviate Docs](https://weaviate.io/developers/weaviate/config-refs/distances)

        If a bucket was embedded using a custom loader, such as `intercom`, the
        additional metadata will be returned in the `loader_metadata` field.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/embeddings/similarity-search",
            body=maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "query": query,
                    "num_of_docs": num_of_docs,
                },
                embedding_similarity_search_params.EmbeddingSimilaritySearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingSimilaritySearchResponse,
        )

    def url(
        self,
        *,
        bucket_name: str,
        url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingResponse:
        """
        Embed website content from a specified URL, including child pages up to 5 levels
        deep within the same domain. The process crawls and loads content from the main
        URL and its linked pages into a Telnyx Cloud Storage bucket. As soon as each
        webpage is added to the bucket, its content is immediately processed for
        embeddings, that can be used for
        [similarity search](https://developers.telnyx.com/api-reference/embeddings/search-for-documents)
        and [clustering](https://developers.telnyx.com/docs/inference/clusters).

        Args:
          bucket_name: Name of the bucket to store the embeddings. This bucket must already exist.

          url: The URL of the webpage to embed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/ai/embeddings/url",
            body=maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "url": url,
                },
                embedding_url_params.EmbeddingURLParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )


class AsyncEmbeddingsResource(AsyncAPIResource):
    @cached_property
    def buckets(self) -> AsyncBucketsResource:
        return AsyncBucketsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEmbeddingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmbeddingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmbeddingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncEmbeddingsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        bucket_name: str,
        document_chunk_overlap_size: int | Omit = omit,
        document_chunk_size: int | Omit = omit,
        embedding_model: Literal["thenlper/gte-large", "intfloat/multilingual-e5-large"] | Omit = omit,
        loader: Literal["default", "intercom"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingResponse:
        """Perform embedding on a Telnyx Storage Bucket using the a embedding model.

        The
        current supported file types are:

        - PDF
        - HTML
        - txt/unstructured text files
        - json
        - csv
        - audio / video (mp3, mp4, mpeg, mpga, m4a, wav, or webm ) - Max of 100mb file
          size.

        Any files not matching the above types will be attempted to be embedded as
        unstructured text.

        This process can be slow, so it runs in the background and the user can check
        the status of the task using the endpoint `/ai/embeddings/{task_id}`.

        **Important Note**: When you update documents in a Telnyx Storage bucket, their
        associated embeddings are automatically kept up to date. If you add or update a
        file, it is automatically embedded. If you delete a file, the embeddings are
        deleted for that particular file.

        You can also specify a custom `loader` param. Currently the only supported
        loader value is `intercom` which loads Intercom article jsons as specified by
        [the Intercom article API](https://developers.intercom.com/docs/references/rest-api/api.intercom.io/Articles/article/)
        This loader will split each article into paragraphs and save additional
        parameters relevant to Intercom docs, such as `article_url` and `heading`. These
        values will be returned by the `/v2/ai/embeddings/similarity-search` endpoint in
        the `loader_metadata` field.

        Args:
          embedding_model: Supported models to vectorize and embed documents.

          loader: Supported types of custom document loaders for embeddings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/embeddings",
            body=await async_maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "document_chunk_overlap_size": document_chunk_overlap_size,
                    "document_chunk_size": document_chunk_size,
                    "embedding_model": embedding_model,
                    "loader": loader,
                },
                embedding_create_params.EmbeddingCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )

    async def retrieve(
        self,
        task_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingRetrieveResponse:
        """Check the status of a current embedding task.

        Will be one of the following:

        - `queued` - Task is waiting to be picked up by a worker
        - `processing` - The embedding task is running
        - `success` - Task completed successfully and the bucket is embedded
        - `failure` - Task failed and no files were embedded successfully
        - `partial_success` - Some files were embedded successfully, but at least one
          failed

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not task_id:
            raise ValueError(f"Expected a non-empty value for `task_id` but received {task_id!r}")
        return await self._get(
            f"/ai/embeddings/{task_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingRetrieveResponse,
        )

    async def list(
        self,
        *,
        status: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingListResponse:
        """
        Retrieve tasks for the user that are either `queued`, `processing`, `failed`,
        `success` or `partial_success` based on the query string. Defaults to `queued`
        and `processing`.

        Args:
          status: List of task statuses i.e. `status=queued&status=processing`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/ai/embeddings",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"status": status}, embedding_list_params.EmbeddingListParams),
            ),
            cast_to=EmbeddingListResponse,
        )

    async def similarity_search(
        self,
        *,
        bucket_name: str,
        query: str,
        num_of_docs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingSimilaritySearchResponse:
        """
        Perform a similarity search on a Telnyx Storage Bucket, returning the most
        similar `num_docs` document chunks to the query.

        Currently the only available distance metric is cosine similarity which will
        return a `distance` between 0 and 1. The lower the distance, the more similar
        the returned document chunks are to the query. A `certainty` will also be
        returned, which is a value between 0 and 1 where the higher the certainty, the
        more similar the document. You can read more about Weaviate distance metrics
        here:
        [Weaviate Docs](https://weaviate.io/developers/weaviate/config-refs/distances)

        If a bucket was embedded using a custom loader, such as `intercom`, the
        additional metadata will be returned in the `loader_metadata` field.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/embeddings/similarity-search",
            body=await async_maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "query": query,
                    "num_of_docs": num_of_docs,
                },
                embedding_similarity_search_params.EmbeddingSimilaritySearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingSimilaritySearchResponse,
        )

    async def url(
        self,
        *,
        bucket_name: str,
        url: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmbeddingResponse:
        """
        Embed website content from a specified URL, including child pages up to 5 levels
        deep within the same domain. The process crawls and loads content from the main
        URL and its linked pages into a Telnyx Cloud Storage bucket. As soon as each
        webpage is added to the bucket, its content is immediately processed for
        embeddings, that can be used for
        [similarity search](https://developers.telnyx.com/api-reference/embeddings/search-for-documents)
        and [clustering](https://developers.telnyx.com/docs/inference/clusters).

        Args:
          bucket_name: Name of the bucket to store the embeddings. This bucket must already exist.

          url: The URL of the webpage to embed

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/ai/embeddings/url",
            body=await async_maybe_transform(
                {
                    "bucket_name": bucket_name,
                    "url": url,
                },
                embedding_url_params.EmbeddingURLParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=EmbeddingResponse,
        )


class EmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = to_raw_response_wrapper(
            embeddings.create,
        )
        self.retrieve = to_raw_response_wrapper(
            embeddings.retrieve,
        )
        self.list = to_raw_response_wrapper(
            embeddings.list,
        )
        self.similarity_search = to_raw_response_wrapper(
            embeddings.similarity_search,
        )
        self.url = to_raw_response_wrapper(
            embeddings.url,
        )

    @cached_property
    def buckets(self) -> BucketsResourceWithRawResponse:
        return BucketsResourceWithRawResponse(self._embeddings.buckets)


class AsyncEmbeddingsResourceWithRawResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = async_to_raw_response_wrapper(
            embeddings.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            embeddings.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            embeddings.list,
        )
        self.similarity_search = async_to_raw_response_wrapper(
            embeddings.similarity_search,
        )
        self.url = async_to_raw_response_wrapper(
            embeddings.url,
        )

    @cached_property
    def buckets(self) -> AsyncBucketsResourceWithRawResponse:
        return AsyncBucketsResourceWithRawResponse(self._embeddings.buckets)


class EmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: EmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = to_streamed_response_wrapper(
            embeddings.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            embeddings.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            embeddings.list,
        )
        self.similarity_search = to_streamed_response_wrapper(
            embeddings.similarity_search,
        )
        self.url = to_streamed_response_wrapper(
            embeddings.url,
        )

    @cached_property
    def buckets(self) -> BucketsResourceWithStreamingResponse:
        return BucketsResourceWithStreamingResponse(self._embeddings.buckets)


class AsyncEmbeddingsResourceWithStreamingResponse:
    def __init__(self, embeddings: AsyncEmbeddingsResource) -> None:
        self._embeddings = embeddings

        self.create = async_to_streamed_response_wrapper(
            embeddings.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            embeddings.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            embeddings.list,
        )
        self.similarity_search = async_to_streamed_response_wrapper(
            embeddings.similarity_search,
        )
        self.url = async_to_streamed_response_wrapper(
            embeddings.url,
        )

    @cached_property
    def buckets(self) -> AsyncBucketsResourceWithStreamingResponse:
        return AsyncBucketsResourceWithStreamingResponse(self._embeddings.buckets)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["GcsConfigurationDataParam"]


class GcsConfigurationDataParam(TypedDict, total=False):
    backend: Required[Literal["gcs"]]
    """Storage backend type"""

    bucket: str
    """Name of the bucket to be used to store recording files."""

    credentials: str
    """
    Opaque credential token used to authenticate and authorize with storage
    provider.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["VerifyVerificationCodeResponse", "Data"]


class Data(BaseModel):
    phone_number: str
    """+E164 formatted phone number."""

    response_code: Literal["accepted", "rejected"]
    """Identifies if the verification code has been accepted or rejected."""


class VerifyVerificationCodeResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .default_gateway_create_params import DefaultGatewayCreateParams as DefaultGatewayCreateParams
from .default_gateway_create_response import DefaultGatewayCreateResponse as DefaultGatewayCreateResponse
from .default_gateway_delete_response import DefaultGatewayDeleteResponse as DefaultGatewayDeleteResponse
from .default_gateway_retrieve_response import DefaultGatewayRetrieveResponse as DefaultGatewayRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.rooms import action_refresh_client_token_params, action_generate_join_client_token_params
from ..._base_client import make_request_options
from ...types.rooms.action_refresh_client_token_response import ActionRefreshClientTokenResponse
from ...types.rooms.action_generate_join_client_token_response import ActionGenerateJoinClientTokenResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def generate_join_client_token(
        self,
        room_id: str,
        *,
        refresh_token_ttl_secs: int | Omit = omit,
        token_ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionGenerateJoinClientTokenResponse:
        """Synchronously create an Client Token to join a Room.

        Client Token is necessary
        to join a Telnyx Room. Client Token will expire after `token_ttl_secs`, a
        Refresh Token is also provided to refresh a Client Token, the Refresh Token
        expires after `refresh_token_ttl_secs`.

        Args:
          refresh_token_ttl_secs: The time to live in seconds of the Refresh Token, after that time the Refresh
              Token is invalid and can't be used to refresh Client Token.

          token_ttl_secs: The time to live in seconds of the Client Token, after that time the Client
              Token is invalid and can't be used to join a Room.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return self._post(
            f"/rooms/{room_id}/actions/generate_join_client_token",
            body=maybe_transform(
                {
                    "refresh_token_ttl_secs": refresh_token_ttl_secs,
                    "token_ttl_secs": token_ttl_secs,
                },
                action_generate_join_client_token_params.ActionGenerateJoinClientTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionGenerateJoinClientTokenResponse,
        )

    def refresh_client_token(
        self,
        room_id: str,
        *,
        refresh_token: str,
        token_ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRefreshClientTokenResponse:
        """Synchronously refresh an Client Token to join a Room.

        Client Token is necessary
        to join a Telnyx Room. Client Token will expire after `token_ttl_secs`.

        Args:
          token_ttl_secs: The time to live in seconds of the Client Token, after that time the Client
              Token is invalid and can't be used to join a Room.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return self._post(
            f"/rooms/{room_id}/actions/refresh_client_token",
            body=maybe_transform(
                {
                    "refresh_token": refresh_token,
                    "token_ttl_secs": token_ttl_secs,
                },
                action_refresh_client_token_params.ActionRefreshClientTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRefreshClientTokenResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def generate_join_client_token(
        self,
        room_id: str,
        *,
        refresh_token_ttl_secs: int | Omit = omit,
        token_ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionGenerateJoinClientTokenResponse:
        """Synchronously create an Client Token to join a Room.

        Client Token is necessary
        to join a Telnyx Room. Client Token will expire after `token_ttl_secs`, a
        Refresh Token is also provided to refresh a Client Token, the Refresh Token
        expires after `refresh_token_ttl_secs`.

        Args:
          refresh_token_ttl_secs: The time to live in seconds of the Refresh Token, after that time the Refresh
              Token is invalid and can't be used to refresh Client Token.

          token_ttl_secs: The time to live in seconds of the Client Token, after that time the Client
              Token is invalid and can't be used to join a Room.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return await self._post(
            f"/rooms/{room_id}/actions/generate_join_client_token",
            body=await async_maybe_transform(
                {
                    "refresh_token_ttl_secs": refresh_token_ttl_secs,
                    "token_ttl_secs": token_ttl_secs,
                },
                action_generate_join_client_token_params.ActionGenerateJoinClientTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionGenerateJoinClientTokenResponse,
        )

    async def refresh_client_token(
        self,
        room_id: str,
        *,
        refresh_token: str,
        token_ttl_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionRefreshClientTokenResponse:
        """Synchronously refresh an Client Token to join a Room.

        Client Token is necessary
        to join a Telnyx Room. Client Token will expire after `token_ttl_secs`.

        Args:
          token_ttl_secs: The time to live in seconds of the Client Token, after that time the Client
              Token is invalid and can't be used to join a Room.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not room_id:
            raise ValueError(f"Expected a non-empty value for `room_id` but received {room_id!r}")
        return await self._post(
            f"/rooms/{room_id}/actions/refresh_client_token",
            body=await async_maybe_transform(
                {
                    "refresh_token": refresh_token,
                    "token_ttl_secs": token_ttl_secs,
                },
                action_refresh_client_token_params.ActionRefreshClientTokenParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionRefreshClientTokenResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.generate_join_client_token = to_raw_response_wrapper(
            actions.generate_join_client_token,
        )
        self.refresh_client_token = to_raw_response_wrapper(
            actions.refresh_client_token,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.generate_join_client_token = async_to_raw_response_wrapper(
            actions.generate_join_client_token,
        )
        self.refresh_client_token = async_to_raw_response_wrapper(
            actions.refresh_client_token,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.generate_join_client_token = to_streamed_response_wrapper(
            actions.generate_join_client_token,
        )
        self.refresh_client_token = to_streamed_response_wrapper(
            actions.refresh_client_token,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.generate_join_client_token = async_to_streamed_response_wrapper(
            actions.generate_join_client_token,
        )
        self.refresh_client_token = async_to_streamed_response_wrapper(
            actions.refresh_client_token,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ReportListMdrsParams"]


class ReportListMdrsParams(TypedDict, total=False):
    id: str
    """Message uuid"""

    cld: str
    """Destination number"""

    cli: str
    """Origination number"""

    direction: Literal["INBOUND", "OUTBOUND"]
    """Direction (inbound or outbound)"""

    end_date: str
    """Pagination end date"""

    message_type: Literal["SMS", "MMS"]
    """Type of message"""

    profile: str
    """Name of the profile"""

    start_date: str
    """Pagination start date"""

    status: Literal["GW_TIMEOUT", "DELIVERED", "DLR_UNCONFIRMED", "DLR_TIMEOUT", "RECEIVED", "GW_REJECT", "FAILED"]
    """Message status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...types import (
    verification_trigger_sms_params,
    verification_trigger_call_params,
    verification_trigger_flashcall_params,
)
from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .by_phone_number.by_phone_number import (
    ByPhoneNumberResource,
    AsyncByPhoneNumberResource,
    ByPhoneNumberResourceWithRawResponse,
    AsyncByPhoneNumberResourceWithRawResponse,
    ByPhoneNumberResourceWithStreamingResponse,
    AsyncByPhoneNumberResourceWithStreamingResponse,
)
from ...types.create_verification_response import CreateVerificationResponse
from ...types.verification_retrieve_response import VerificationRetrieveResponse

__all__ = ["VerificationsResource", "AsyncVerificationsResource"]


class VerificationsResource(SyncAPIResource):
    @cached_property
    def by_phone_number(self) -> ByPhoneNumberResource:
        return ByPhoneNumberResource(self._client)

    @cached_property
    def actions(self) -> ActionsResource:
        return ActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> VerificationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VerificationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VerificationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VerificationsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        verification_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRetrieveResponse:
        """
        Retrieve verification

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verification_id:
            raise ValueError(f"Expected a non-empty value for `verification_id` but received {verification_id!r}")
        return self._get(
            f"/verifications/{verification_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRetrieveResponse,
        )

    def trigger_call(
        self,
        *,
        phone_number: str,
        verify_profile_id: str,
        custom_code: Optional[str] | Omit = omit,
        extension: Optional[str] | Omit = omit,
        timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreateVerificationResponse:
        """
        Trigger Call verification

        Args:
          phone_number: +E164 formatted phone number.

          verify_profile_id: The identifier of the associated Verify profile.

          custom_code: Send a self-generated numeric code to the end-user

          extension: Optional extension to dial after call is answered using DTMF digits. Valid
              digits are 0-9, A-D, \\**, and #. Pauses can be added using w (0.5s) and W (1s).

          timeout_secs: The number of seconds the verification code is valid for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/verifications/call",
            body=maybe_transform(
                {
                    "phone_number": phone_number,
                    "verify_profile_id": verify_profile_id,
                    "custom_code": custom_code,
                    "extension": extension,
                    "timeout_secs": timeout_secs,
                },
                verification_trigger_call_params.VerificationTriggerCallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateVerificationResponse,
        )

    def trigger_flashcall(
        self,
        *,
        phone_number: str,
        verify_profile_id: str,
        timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreateVerificationResponse:
        """
        Trigger Flash call verification

        Args:
          phone_number: +E164 formatted phone number.

          verify_profile_id: The identifier of the associated Verify profile.

          timeout_secs: The number of seconds the verification code is valid for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/verifications/flashcall",
            body=maybe_transform(
                {
                    "phone_number": phone_number,
                    "verify_profile_id": verify_profile_id,
                    "timeout_secs": timeout_secs,
                },
                verification_trigger_flashcall_params.VerificationTriggerFlashcallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateVerificationResponse,
        )

    def trigger_sms(
        self,
        *,
        phone_number: str,
        verify_profile_id: str,
        custom_code: Optional[str] | Omit = omit,
        timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreateVerificationResponse:
        """
        Trigger SMS verification

        Args:
          phone_number: +E164 formatted phone number.

          verify_profile_id: The identifier of the associated Verify profile.

          custom_code: Send a self-generated numeric code to the end-user

          timeout_secs: The number of seconds the verification code is valid for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/verifications/sms",
            body=maybe_transform(
                {
                    "phone_number": phone_number,
                    "verify_profile_id": verify_profile_id,
                    "custom_code": custom_code,
                    "timeout_secs": timeout_secs,
                },
                verification_trigger_sms_params.VerificationTriggerSMSParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateVerificationResponse,
        )


class AsyncVerificationsResource(AsyncAPIResource):
    @cached_property
    def by_phone_number(self) -> AsyncByPhoneNumberResource:
        return AsyncByPhoneNumberResource(self._client)

    @cached_property
    def actions(self) -> AsyncActionsResource:
        return AsyncActionsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncVerificationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVerificationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVerificationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVerificationsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        verification_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRetrieveResponse:
        """
        Retrieve verification

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not verification_id:
            raise ValueError(f"Expected a non-empty value for `verification_id` but received {verification_id!r}")
        return await self._get(
            f"/verifications/{verification_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRetrieveResponse,
        )

    async def trigger_call(
        self,
        *,
        phone_number: str,
        verify_profile_id: str,
        custom_code: Optional[str] | Omit = omit,
        extension: Optional[str] | Omit = omit,
        timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreateVerificationResponse:
        """
        Trigger Call verification

        Args:
          phone_number: +E164 formatted phone number.

          verify_profile_id: The identifier of the associated Verify profile.

          custom_code: Send a self-generated numeric code to the end-user

          extension: Optional extension to dial after call is answered using DTMF digits. Valid
              digits are 0-9, A-D, \\**, and #. Pauses can be added using w (0.5s) and W (1s).

          timeout_secs: The number of seconds the verification code is valid for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/verifications/call",
            body=await async_maybe_transform(
                {
                    "phone_number": phone_number,
                    "verify_profile_id": verify_profile_id,
                    "custom_code": custom_code,
                    "extension": extension,
                    "timeout_secs": timeout_secs,
                },
                verification_trigger_call_params.VerificationTriggerCallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateVerificationResponse,
        )

    async def trigger_flashcall(
        self,
        *,
        phone_number: str,
        verify_profile_id: str,
        timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreateVerificationResponse:
        """
        Trigger Flash call verification

        Args:
          phone_number: +E164 formatted phone number.

          verify_profile_id: The identifier of the associated Verify profile.

          timeout_secs: The number of seconds the verification code is valid for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/verifications/flashcall",
            body=await async_maybe_transform(
                {
                    "phone_number": phone_number,
                    "verify_profile_id": verify_profile_id,
                    "timeout_secs": timeout_secs,
                },
                verification_trigger_flashcall_params.VerificationTriggerFlashcallParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateVerificationResponse,
        )

    async def trigger_sms(
        self,
        *,
        phone_number: str,
        verify_profile_id: str,
        custom_code: Optional[str] | Omit = omit,
        timeout_secs: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CreateVerificationResponse:
        """
        Trigger SMS verification

        Args:
          phone_number: +E164 formatted phone number.

          verify_profile_id: The identifier of the associated Verify profile.

          custom_code: Send a self-generated numeric code to the end-user

          timeout_secs: The number of seconds the verification code is valid for.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/verifications/sms",
            body=await async_maybe_transform(
                {
                    "phone_number": phone_number,
                    "verify_profile_id": verify_profile_id,
                    "custom_code": custom_code,
                    "timeout_secs": timeout_secs,
                },
                verification_trigger_sms_params.VerificationTriggerSMSParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateVerificationResponse,
        )


class VerificationsResourceWithRawResponse:
    def __init__(self, verifications: VerificationsResource) -> None:
        self._verifications = verifications

        self.retrieve = to_raw_response_wrapper(
            verifications.retrieve,
        )
        self.trigger_call = to_raw_response_wrapper(
            verifications.trigger_call,
        )
        self.trigger_flashcall = to_raw_response_wrapper(
            verifications.trigger_flashcall,
        )
        self.trigger_sms = to_raw_response_wrapper(
            verifications.trigger_sms,
        )

    @cached_property
    def by_phone_number(self) -> ByPhoneNumberResourceWithRawResponse:
        return ByPhoneNumberResourceWithRawResponse(self._verifications.by_phone_number)

    @cached_property
    def actions(self) -> ActionsResourceWithRawResponse:
        return ActionsResourceWithRawResponse(self._verifications.actions)


class AsyncVerificationsResourceWithRawResponse:
    def __init__(self, verifications: AsyncVerificationsResource) -> None:
        self._verifications = verifications

        self.retrieve = async_to_raw_response_wrapper(
            verifications.retrieve,
        )
        self.trigger_call = async_to_raw_response_wrapper(
            verifications.trigger_call,
        )
        self.trigger_flashcall = async_to_raw_response_wrapper(
            verifications.trigger_flashcall,
        )
        self.trigger_sms = async_to_raw_response_wrapper(
            verifications.trigger_sms,
        )

    @cached_property
    def by_phone_number(self) -> AsyncByPhoneNumberResourceWithRawResponse:
        return AsyncByPhoneNumberResourceWithRawResponse(self._verifications.by_phone_number)

    @cached_property
    def actions(self) -> AsyncActionsResourceWithRawResponse:
        return AsyncActionsResourceWithRawResponse(self._verifications.actions)


class VerificationsResourceWithStreamingResponse:
    def __init__(self, verifications: VerificationsResource) -> None:
        self._verifications = verifications

        self.retrieve = to_streamed_response_wrapper(
            verifications.retrieve,
        )
        self.trigger_call = to_streamed_response_wrapper(
            verifications.trigger_call,
        )
        self.trigger_flashcall = to_streamed_response_wrapper(
            verifications.trigger_flashcall,
        )
        self.trigger_sms = to_streamed_response_wrapper(
            verifications.trigger_sms,
        )

    @cached_property
    def by_phone_number(self) -> ByPhoneNumberResourceWithStreamingResponse:
        return ByPhoneNumberResourceWithStreamingResponse(self._verifications.by_phone_number)

    @cached_property
    def actions(self) -> ActionsResourceWithStreamingResponse:
        return ActionsResourceWithStreamingResponse(self._verifications.actions)


class AsyncVerificationsResourceWithStreamingResponse:
    def __init__(self, verifications: AsyncVerificationsResource) -> None:
        self._verifications = verifications

        self.retrieve = async_to_streamed_response_wrapper(
            verifications.retrieve,
        )
        self.trigger_call = async_to_streamed_response_wrapper(
            verifications.trigger_call,
        )
        self.trigger_flashcall = async_to_streamed_response_wrapper(
            verifications.trigger_flashcall,
        )
        self.trigger_sms = async_to_streamed_response_wrapper(
            verifications.trigger_sms,
        )

    @cached_property
    def by_phone_number(self) -> AsyncByPhoneNumberResourceWithStreamingResponse:
        return AsyncByPhoneNumberResourceWithStreamingResponse(self._verifications.by_phone_number)

    @cached_property
    def actions(self) -> AsyncActionsResourceWithStreamingResponse:
        return AsyncActionsResourceWithStreamingResponse(self._verifications.actions)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import number_order_list_params, number_order_create_params, number_order_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.number_order_list_response import NumberOrderListResponse
from ..types.number_order_create_response import NumberOrderCreateResponse
from ..types.number_order_update_response import NumberOrderUpdateResponse
from ..types.number_order_retrieve_response import NumberOrderRetrieveResponse
from ..types.update_regulatory_requirement_param import UpdateRegulatoryRequirementParam

__all__ = ["NumberOrdersResource", "AsyncNumberOrdersResource"]


class NumberOrdersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NumberOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return NumberOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NumberOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return NumberOrdersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        billing_group_id: str | Omit = omit,
        connection_id: str | Omit = omit,
        customer_reference: str | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        phone_numbers: Iterable[number_order_create_params.PhoneNumber] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderCreateResponse:
        """
        Creates a phone number order.

        Args:
          billing_group_id: Identifies the billing group associated with the phone number.

          connection_id: Identifies the connection associated with this phone number.

          customer_reference: A customer reference string for customer look ups.

          messaging_profile_id: Identifies the messaging profile associated with the phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/number_orders",
            body=maybe_transform(
                {
                    "billing_group_id": billing_group_id,
                    "connection_id": connection_id,
                    "customer_reference": customer_reference,
                    "messaging_profile_id": messaging_profile_id,
                    "phone_numbers": phone_numbers,
                },
                number_order_create_params.NumberOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderCreateResponse,
        )

    def retrieve(
        self,
        number_order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderRetrieveResponse:
        """
        Get an existing phone number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_id:
            raise ValueError(f"Expected a non-empty value for `number_order_id` but received {number_order_id!r}")
        return self._get(
            f"/number_orders/{number_order_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderRetrieveResponse,
        )

    def update(
        self,
        number_order_id: str,
        *,
        customer_reference: str | Omit = omit,
        regulatory_requirements: Iterable[UpdateRegulatoryRequirementParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderUpdateResponse:
        """
        Updates a phone number order.

        Args:
          customer_reference: A customer reference string for customer look ups.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_id:
            raise ValueError(f"Expected a non-empty value for `number_order_id` but received {number_order_id!r}")
        return self._patch(
            f"/number_orders/{number_order_id}",
            body=maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "regulatory_requirements": regulatory_requirements,
                },
                number_order_update_params.NumberOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderUpdateResponse,
        )

    def list(
        self,
        *,
        filter: number_order_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[NumberOrderListResponse]:
        """
        Get a paginated list of number orders.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[created_at], filter[phone_numbers_count], filter[customer_reference],
              filter[requirements_met]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/number_orders",
            page=SyncDefaultFlatPagination[NumberOrderListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    number_order_list_params.NumberOrderListParams,
                ),
            ),
            model=NumberOrderListResponse,
        )


class AsyncNumberOrdersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNumberOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNumberOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNumberOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncNumberOrdersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        billing_group_id: str | Omit = omit,
        connection_id: str | Omit = omit,
        customer_reference: str | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        phone_numbers: Iterable[number_order_create_params.PhoneNumber] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderCreateResponse:
        """
        Creates a phone number order.

        Args:
          billing_group_id: Identifies the billing group associated with the phone number.

          connection_id: Identifies the connection associated with this phone number.

          customer_reference: A customer reference string for customer look ups.

          messaging_profile_id: Identifies the messaging profile associated with the phone number.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/number_orders",
            body=await async_maybe_transform(
                {
                    "billing_group_id": billing_group_id,
                    "connection_id": connection_id,
                    "customer_reference": customer_reference,
                    "messaging_profile_id": messaging_profile_id,
                    "phone_numbers": phone_numbers,
                },
                number_order_create_params.NumberOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderCreateResponse,
        )

    async def retrieve(
        self,
        number_order_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderRetrieveResponse:
        """
        Get an existing phone number order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_id:
            raise ValueError(f"Expected a non-empty value for `number_order_id` but received {number_order_id!r}")
        return await self._get(
            f"/number_orders/{number_order_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderRetrieveResponse,
        )

    async def update(
        self,
        number_order_id: str,
        *,
        customer_reference: str | Omit = omit,
        regulatory_requirements: Iterable[UpdateRegulatoryRequirementParam] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> NumberOrderUpdateResponse:
        """
        Updates a phone number order.

        Args:
          customer_reference: A customer reference string for customer look ups.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not number_order_id:
            raise ValueError(f"Expected a non-empty value for `number_order_id` but received {number_order_id!r}")
        return await self._patch(
            f"/number_orders/{number_order_id}",
            body=await async_maybe_transform(
                {
                    "customer_reference": customer_reference,
                    "regulatory_requirements": regulatory_requirements,
                },
                number_order_update_params.NumberOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NumberOrderUpdateResponse,
        )

    def list(
        self,
        *,
        filter: number_order_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[NumberOrderListResponse, AsyncDefaultFlatPagination[NumberOrderListResponse]]:
        """
        Get a paginated list of number orders.

        Args:
          filter: Consolidated filter parameter (deepObject style). Originally: filter[status],
              filter[created_at], filter[phone_numbers_count], filter[customer_reference],
              filter[requirements_met]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/number_orders",
            page=AsyncDefaultFlatPagination[NumberOrderListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    number_order_list_params.NumberOrderListParams,
                ),
            ),
            model=NumberOrderListResponse,
        )


class NumberOrdersResourceWithRawResponse:
    def __init__(self, number_orders: NumberOrdersResource) -> None:
        self._number_orders = number_orders

        self.create = to_raw_response_wrapper(
            number_orders.create,
        )
        self.retrieve = to_raw_response_wrapper(
            number_orders.retrieve,
        )
        self.update = to_raw_response_wrapper(
            number_orders.update,
        )
        self.list = to_raw_response_wrapper(
            number_orders.list,
        )


class AsyncNumberOrdersResourceWithRawResponse:
    def __init__(self, number_orders: AsyncNumberOrdersResource) -> None:
        self._number_orders = number_orders

        self.create = async_to_raw_response_wrapper(
            number_orders.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            number_orders.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            number_orders.update,
        )
        self.list = async_to_raw_response_wrapper(
            number_orders.list,
        )


class NumberOrdersResourceWithStreamingResponse:
    def __init__(self, number_orders: NumberOrdersResource) -> None:
        self._number_orders = number_orders

        self.create = to_streamed_response_wrapper(
            number_orders.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            number_orders.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            number_orders.update,
        )
        self.list = to_streamed_response_wrapper(
            number_orders.list,
        )


class AsyncNumberOrdersResourceWithStreamingResponse:
    def __init__(self, number_orders: AsyncNumberOrdersResource) -> None:
        self._number_orders = number_orders

        self.create = async_to_streamed_response_wrapper(
            number_orders.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            number_orders.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            number_orders.update,
        )
        self.list = async_to_streamed_response_wrapper(
            number_orders.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["FineTuningResource", "AsyncFineTuningResource"]


class FineTuningResource(SyncAPIResource):
    @cached_property
    def jobs(self) -> JobsResource:
        return JobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> FineTuningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return FineTuningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FineTuningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return FineTuningResourceWithStreamingResponse(self)


class AsyncFineTuningResource(AsyncAPIResource):
    @cached_property
    def jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFineTuningResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFineTuningResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFineTuningResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncFineTuningResourceWithStreamingResponse(self)


class FineTuningResourceWithRawResponse:
    def __init__(self, fine_tuning: FineTuningResource) -> None:
        self._fine_tuning = fine_tuning

    @cached_property
    def jobs(self) -> JobsResourceWithRawResponse:
        return JobsResourceWithRawResponse(self._fine_tuning.jobs)


class AsyncFineTuningResourceWithRawResponse:
    def __init__(self, fine_tuning: AsyncFineTuningResource) -> None:
        self._fine_tuning = fine_tuning

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._fine_tuning.jobs)


class FineTuningResourceWithStreamingResponse:
    def __init__(self, fine_tuning: FineTuningResource) -> None:
        self._fine_tuning = fine_tuning

    @cached_property
    def jobs(self) -> JobsResourceWithStreamingResponse:
        return JobsResourceWithStreamingResponse(self._fine_tuning.jobs)


class AsyncFineTuningResourceWithStreamingResponse:
    def __init__(self, fine_tuning: AsyncFineTuningResource) -> None:
        self._fine_tuning = fine_tuning

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._fine_tuning.jobs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .cnam_listing import CnamListing
from .call_recording import CallRecording
from .media_features import MediaFeatures
from .call_forwarding import CallForwarding

__all__ = ["PhoneNumberWithVoiceSettings", "Emergency"]


class Emergency(BaseModel):
    """The emergency services settings for a phone number."""

    emergency_address_id: Optional[str] = None
    """Identifies the address to be used with emergency services."""

    emergency_enabled: Optional[bool] = None
    """Allows you to enable or disable emergency services on the phone number.

    In order to enable emergency services, you must also set an
    emergency_address_id.
    """

    emergency_status: Optional[
        Literal["disabled", "active", "provisioning", "deprovisioning", "provisioning-failed"]
    ] = None
    """Represents the state of the number regarding emergency activation."""


class PhoneNumberWithVoiceSettings(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    call_forwarding: Optional[CallForwarding] = None
    """The call forwarding settings for a phone number."""

    call_recording: Optional[CallRecording] = None
    """The call recording settings for a phone number."""

    cnam_listing: Optional[CnamListing] = None
    """The CNAM listing settings for a phone number."""

    connection_id: Optional[str] = None
    """Identifies the connection associated with this phone number."""

    customer_reference: Optional[str] = None
    """A customer reference string for customer look ups."""

    emergency: Optional[Emergency] = None
    """The emergency services settings for a phone number."""

    inbound_call_screening: Optional[Literal["disabled", "reject_calls", "flag_calls"]] = None
    """
    The inbound_call_screening setting is a phone number configuration option
    variable that allows users to configure their settings to block or flag
    fraudulent calls. It can be set to disabled, reject_calls, or flag_calls. This
    feature has an additional per-number monthly cost associated with it.
    """

    media_features: Optional[MediaFeatures] = None
    """The media features settings for a phone number."""

    phone_number: Optional[str] = None
    """The phone number in +E164 format."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    tech_prefix_enabled: Optional[bool] = None
    """Controls whether a tech prefix is enabled for this phone number."""

    translated_number: Optional[str] = None
    """
    This field allows you to rewrite the destination number of an inbound call
    before the call is routed to you. The value of this field may be any
    alphanumeric value, and the value will replace the number originally dialed.
    """

    usage_payment_method: Optional[Literal["pay-per-minute", "channel"]] = None
    """
    Controls whether a number is billed per minute or uses your concurrent channels.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .texml import (
    TexmlResource,
    AsyncTexmlResource,
    TexmlResourceWithRawResponse,
    AsyncTexmlResourceWithRawResponse,
    TexmlResourceWithStreamingResponse,
    AsyncTexmlResourceWithStreamingResponse,
)
from .accounts import (
    AccountsResource,
    AsyncAccountsResource,
    AccountsResourceWithRawResponse,
    AsyncAccountsResourceWithRawResponse,
    AccountsResourceWithStreamingResponse,
    AsyncAccountsResourceWithStreamingResponse,
)

__all__ = [
    "AccountsResource",
    "AsyncAccountsResource",
    "AccountsResourceWithRawResponse",
    "AsyncAccountsResourceWithRawResponse",
    "AccountsResourceWithStreamingResponse",
    "AsyncAccountsResourceWithStreamingResponse",
    "TexmlResource",
    "AsyncTexmlResource",
    "TexmlResourceWithRawResponse",
    "AsyncTexmlResourceWithRawResponse",
    "TexmlResourceWithStreamingResponse",
    "AsyncTexmlResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["OAuthRegisterParams"]


class OAuthRegisterParams(TypedDict, total=False):
    client_name: str
    """Human-readable string name of the client to be presented to the end-user"""

    grant_types: List[Literal["authorization_code", "client_credentials", "refresh_token"]]
    """Array of OAuth 2.0 grant type strings that the client may use"""

    logo_uri: str
    """URL of the client logo"""

    policy_uri: str
    """URL of the client's privacy policy"""

    redirect_uris: SequenceNotStr[str]
    """Array of redirection URI strings for use in redirect-based flows"""

    response_types: SequenceNotStr[str]
    """Array of the OAuth 2.0 response type strings that the client may use"""

    scope: str
    """Space-separated string of scope values that the client may use"""

    token_endpoint_auth_method: Literal["none", "client_secret_basic", "client_secret_post"]
    """Authentication method for the token endpoint"""

    tos_uri: str
    """URL of the client's terms of service"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.porting_orders import (
    PortingPhoneNumberExtension,
    PhoneNumberExtensionCreateResponse,
    PhoneNumberExtensionDeleteResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhoneNumberExtensions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        phone_number_extension = client.porting_orders.phone_number_extensions.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": 10,
                    "start_at": 1,
                }
            ],
            extension_range={
                "end_at": 10,
                "start_at": 1,
            },
            porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
        )
        assert_matches_type(PhoneNumberExtensionCreateResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_extensions.with_raw_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": 10,
                    "start_at": 1,
                }
            ],
            extension_range={
                "end_at": 10,
                "start_at": 1,
            },
            porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_extension = response.parse()
        assert_matches_type(PhoneNumberExtensionCreateResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_extensions.with_streaming_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": 10,
                    "start_at": 1,
                }
            ],
            extension_range={
                "end_at": 10,
                "start_at": 1,
            },
            porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_extension = response.parse()
            assert_matches_type(PhoneNumberExtensionCreateResponse, phone_number_extension, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            client.porting_orders.phone_number_extensions.with_raw_response.create(
                porting_order_id="",
                activation_ranges=[
                    {
                        "end_at": 10,
                        "start_at": 1,
                    }
                ],
                extension_range={
                    "end_at": 10,
                    "start_at": 1,
                },
                porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        phone_number_extension = client.porting_orders.phone_number_extensions.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            SyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        phone_number_extension = client.porting_orders.phone_number_extensions.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={"porting_phone_number_id": "04f8f1b9-310c-4a3c-963e-7dfc54765140"},
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            SyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_extensions.with_raw_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_extension = response.parse()
        assert_matches_type(
            SyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_extensions.with_streaming_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_extension = response.parse()
            assert_matches_type(
                SyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            client.porting_orders.phone_number_extensions.with_raw_response.list(
                porting_order_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Telnyx) -> None:
        phone_number_extension = client.porting_orders.phone_number_extensions.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PhoneNumberExtensionDeleteResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Telnyx) -> None:
        response = client.porting_orders.phone_number_extensions.with_raw_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_extension = response.parse()
        assert_matches_type(PhoneNumberExtensionDeleteResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Telnyx) -> None:
        with client.porting_orders.phone_number_extensions.with_streaming_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_extension = response.parse()
            assert_matches_type(PhoneNumberExtensionDeleteResponse, phone_number_extension, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            client.porting_orders.phone_number_extensions.with_raw_response.delete(
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                porting_order_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.porting_orders.phone_number_extensions.with_raw_response.delete(
                id="",
                porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )


class TestAsyncPhoneNumberExtensions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        phone_number_extension = await async_client.porting_orders.phone_number_extensions.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": 10,
                    "start_at": 1,
                }
            ],
            extension_range={
                "end_at": 10,
                "start_at": 1,
            },
            porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
        )
        assert_matches_type(PhoneNumberExtensionCreateResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_extensions.with_raw_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": 10,
                    "start_at": 1,
                }
            ],
            extension_range={
                "end_at": 10,
                "start_at": 1,
            },
            porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_extension = await response.parse()
        assert_matches_type(PhoneNumberExtensionCreateResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_extensions.with_streaming_response.create(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            activation_ranges=[
                {
                    "end_at": 10,
                    "start_at": 1,
                }
            ],
            extension_range={
                "end_at": 10,
                "start_at": 1,
            },
            porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_extension = await response.parse()
            assert_matches_type(PhoneNumberExtensionCreateResponse, phone_number_extension, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            await async_client.porting_orders.phone_number_extensions.with_raw_response.create(
                porting_order_id="",
                activation_ranges=[
                    {
                        "end_at": 10,
                        "start_at": 1,
                    }
                ],
                extension_range={
                    "end_at": 10,
                    "start_at": 1,
                },
                porting_phone_number_id="f24151b6-3389-41d3-8747-7dd8c681e5e2",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        phone_number_extension = await async_client.porting_orders.phone_number_extensions.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        phone_number_extension = await async_client.porting_orders.phone_number_extensions.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            filter={"porting_phone_number_id": "04f8f1b9-310c-4a3c-963e-7dfc54765140"},
            page_number=0,
            page_size=0,
            sort={"value": "created_at"},
        )
        assert_matches_type(
            AsyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_extensions.with_raw_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_extension = await response.parse()
        assert_matches_type(
            AsyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_extensions.with_streaming_response.list(
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_extension = await response.parse()
            assert_matches_type(
                AsyncDefaultFlatPagination[PortingPhoneNumberExtension], phone_number_extension, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            await async_client.porting_orders.phone_number_extensions.with_raw_response.list(
                porting_order_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncTelnyx) -> None:
        phone_number_extension = await async_client.porting_orders.phone_number_extensions.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(PhoneNumberExtensionDeleteResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.porting_orders.phone_number_extensions.with_raw_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        phone_number_extension = await response.parse()
        assert_matches_type(PhoneNumberExtensionDeleteResponse, phone_number_extension, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncTelnyx) -> None:
        async with async_client.porting_orders.phone_number_extensions.with_streaming_response.delete(
            id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            phone_number_extension = await response.parse()
            assert_matches_type(PhoneNumberExtensionDeleteResponse, phone_number_extension, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `porting_order_id` but received ''"):
            await async_client.porting_orders.phone_number_extensions.with_raw_response.delete(
                id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                porting_order_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.porting_orders.phone_number_extensions.with_raw_response.delete(
                id="",
                porting_order_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["VersionRetrieveParams"]


class VersionRetrieveParams(TypedDict, total=False):
    assistant_id: Required[str]

    include_mcp_servers: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, strip_not_given, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.bundle_pricing import user_bundle_list_params, user_bundle_create_params, user_bundle_list_unused_params
from ...types.bundle_pricing.user_bundle import UserBundle
from ...types.bundle_pricing.user_bundle_create_response import UserBundleCreateResponse
from ...types.bundle_pricing.user_bundle_retrieve_response import UserBundleRetrieveResponse
from ...types.bundle_pricing.user_bundle_deactivate_response import UserBundleDeactivateResponse
from ...types.bundle_pricing.user_bundle_list_unused_response import UserBundleListUnusedResponse
from ...types.bundle_pricing.user_bundle_list_resources_response import UserBundleListResourcesResponse

__all__ = ["UserBundlesResource", "AsyncUserBundlesResource"]


class UserBundlesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserBundlesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UserBundlesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserBundlesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UserBundlesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        idempotency_key: str | Omit = omit,
        items: Iterable[user_bundle_create_params.Item] | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleCreateResponse:
        """
        Creates multiple user bundles for the user.

        Args:
          idempotency_key: Idempotency key for the request. Can be any UUID, but should always be unique
              for each request.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._post(
            "/bundle_pricing/user_bundles/bulk",
            body=maybe_transform(
                {
                    "idempotency_key": idempotency_key,
                    "items": items,
                },
                user_bundle_create_params.UserBundleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleCreateResponse,
        )

    def retrieve(
        self,
        user_bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleRetrieveResponse:
        """
        Retrieves a user bundle by its ID.

        Args:
          user_bundle_id: User bundle's ID, this is used to identify the user bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_bundle_id:
            raise ValueError(f"Expected a non-empty value for `user_bundle_id` but received {user_bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get(
            f"/bundle_pricing/user_bundles/{user_bundle_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: user_bundle_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[UserBundle]:
        """
        Get a paginated list of user bundles.

        Args:
          filter: Consolidated filter parameter (deepObject style). Supports filtering by
              country_iso and resource. Examples: filter[country_iso]=US or
              filter[resource]=+15617819942

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get_api_list(
            "/bundle_pricing/user_bundles",
            page=SyncDefaultFlatPagination[UserBundle],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    user_bundle_list_params.UserBundleListParams,
                ),
            ),
            model=UserBundle,
        )

    def deactivate(
        self,
        user_bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleDeactivateResponse:
        """
        Deactivates a user bundle by its ID.

        Args:
          user_bundle_id: User bundle's ID, this is used to identify the user bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_bundle_id:
            raise ValueError(f"Expected a non-empty value for `user_bundle_id` but received {user_bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._delete(
            f"/bundle_pricing/user_bundles/{user_bundle_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleDeactivateResponse,
        )

    def list_resources(
        self,
        user_bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleListResourcesResponse:
        """
        Retrieves the resources of a user bundle by its ID.

        Args:
          user_bundle_id: User bundle's ID, this is used to identify the user bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_bundle_id:
            raise ValueError(f"Expected a non-empty value for `user_bundle_id` but received {user_bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get(
            f"/bundle_pricing/user_bundles/{user_bundle_id}/resources",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleListResourcesResponse,
        )

    def list_unused(
        self,
        *,
        filter: user_bundle_list_unused_params.Filter | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleListUnusedResponse:
        """
        Returns all user bundles that aren't in use.

        Args:
          filter: Consolidated filter parameter (deepObject style). Supports filtering by
              country_iso and resource. Examples: filter[country_iso]=US or
              filter[resource]=+15617819942

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get(
            "/bundle_pricing/user_bundles/unused",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"filter": filter}, user_bundle_list_unused_params.UserBundleListUnusedParams),
            ),
            cast_to=UserBundleListUnusedResponse,
        )


class AsyncUserBundlesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserBundlesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUserBundlesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserBundlesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUserBundlesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        idempotency_key: str | Omit = omit,
        items: Iterable[user_bundle_create_params.Item] | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleCreateResponse:
        """
        Creates multiple user bundles for the user.

        Args:
          idempotency_key: Idempotency key for the request. Can be any UUID, but should always be unique
              for each request.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return await self._post(
            "/bundle_pricing/user_bundles/bulk",
            body=await async_maybe_transform(
                {
                    "idempotency_key": idempotency_key,
                    "items": items,
                },
                user_bundle_create_params.UserBundleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleCreateResponse,
        )

    async def retrieve(
        self,
        user_bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleRetrieveResponse:
        """
        Retrieves a user bundle by its ID.

        Args:
          user_bundle_id: User bundle's ID, this is used to identify the user bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_bundle_id:
            raise ValueError(f"Expected a non-empty value for `user_bundle_id` but received {user_bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return await self._get(
            f"/bundle_pricing/user_bundles/{user_bundle_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: user_bundle_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[UserBundle, AsyncDefaultFlatPagination[UserBundle]]:
        """
        Get a paginated list of user bundles.

        Args:
          filter: Consolidated filter parameter (deepObject style). Supports filtering by
              country_iso and resource. Examples: filter[country_iso]=US or
              filter[resource]=+15617819942

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return self._get_api_list(
            "/bundle_pricing/user_bundles",
            page=AsyncDefaultFlatPagination[UserBundle],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    user_bundle_list_params.UserBundleListParams,
                ),
            ),
            model=UserBundle,
        )

    async def deactivate(
        self,
        user_bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleDeactivateResponse:
        """
        Deactivates a user bundle by its ID.

        Args:
          user_bundle_id: User bundle's ID, this is used to identify the user bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_bundle_id:
            raise ValueError(f"Expected a non-empty value for `user_bundle_id` but received {user_bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return await self._delete(
            f"/bundle_pricing/user_bundles/{user_bundle_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleDeactivateResponse,
        )

    async def list_resources(
        self,
        user_bundle_id: str,
        *,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleListResourcesResponse:
        """
        Retrieves the resources of a user bundle by its ID.

        Args:
          user_bundle_id: User bundle's ID, this is used to identify the user bundle in the API.

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_bundle_id:
            raise ValueError(f"Expected a non-empty value for `user_bundle_id` but received {user_bundle_id!r}")
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return await self._get(
            f"/bundle_pricing/user_bundles/{user_bundle_id}/resources",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserBundleListResourcesResponse,
        )

    async def list_unused(
        self,
        *,
        filter: user_bundle_list_unused_params.Filter | Omit = omit,
        authorization_bearer: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserBundleListUnusedResponse:
        """
        Returns all user bundles that aren't in use.

        Args:
          filter: Consolidated filter parameter (deepObject style). Supports filtering by
              country_iso and resource. Examples: filter[country_iso]=US or
              filter[resource]=+15617819942

          authorization_bearer: Authenticates the request with your Telnyx API V2 KEY

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {**strip_not_given({"authorization_bearer": authorization_bearer}), **(extra_headers or {})}
        return await self._get(
            "/bundle_pricing/user_bundles/unused",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"filter": filter}, user_bundle_list_unused_params.UserBundleListUnusedParams
                ),
            ),
            cast_to=UserBundleListUnusedResponse,
        )


class UserBundlesResourceWithRawResponse:
    def __init__(self, user_bundles: UserBundlesResource) -> None:
        self._user_bundles = user_bundles

        self.create = to_raw_response_wrapper(
            user_bundles.create,
        )
        self.retrieve = to_raw_response_wrapper(
            user_bundles.retrieve,
        )
        self.list = to_raw_response_wrapper(
            user_bundles.list,
        )
        self.deactivate = to_raw_response_wrapper(
            user_bundles.deactivate,
        )
        self.list_resources = to_raw_response_wrapper(
            user_bundles.list_resources,
        )
        self.list_unused = to_raw_response_wrapper(
            user_bundles.list_unused,
        )


class AsyncUserBundlesResourceWithRawResponse:
    def __init__(self, user_bundles: AsyncUserBundlesResource) -> None:
        self._user_bundles = user_bundles

        self.create = async_to_raw_response_wrapper(
            user_bundles.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            user_bundles.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            user_bundles.list,
        )
        self.deactivate = async_to_raw_response_wrapper(
            user_bundles.deactivate,
        )
        self.list_resources = async_to_raw_response_wrapper(
            user_bundles.list_resources,
        )
        self.list_unused = async_to_raw_response_wrapper(
            user_bundles.list_unused,
        )


class UserBundlesResourceWithStreamingResponse:
    def __init__(self, user_bundles: UserBundlesResource) -> None:
        self._user_bundles = user_bundles

        self.create = to_streamed_response_wrapper(
            user_bundles.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            user_bundles.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            user_bundles.list,
        )
        self.deactivate = to_streamed_response_wrapper(
            user_bundles.deactivate,
        )
        self.list_resources = to_streamed_response_wrapper(
            user_bundles.list_resources,
        )
        self.list_unused = to_streamed_response_wrapper(
            user_bundles.list_unused,
        )


class AsyncUserBundlesResourceWithStreamingResponse:
    def __init__(self, user_bundles: AsyncUserBundlesResource) -> None:
        self._user_bundles = user_bundles

        self.create = async_to_streamed_response_wrapper(
            user_bundles.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            user_bundles.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            user_bundles.list,
        )
        self.deactivate = async_to_streamed_response_wrapper(
            user_bundles.deactivate,
        )
        self.list_resources = async_to_streamed_response_wrapper(
            user_bundles.list_resources,
        )
        self.list_unused = async_to_streamed_response_wrapper(
            user_bundles.list_unused,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ActionRequirementInitiateParams", "Params"]


class ActionRequirementInitiateParams(TypedDict, total=False):
    porting_order_id: Required[str]

    params: Required[Params]
    """
    Required information for initiating the action requirement for AU ID
    verification.
    """


class Params(TypedDict, total=False):
    """
    Required information for initiating the action requirement for AU ID verification.
    """

    first_name: Required[str]
    """The first name of the person that will perform the verification flow."""

    last_name: Required[str]
    """The last name of the person that will perform the verification flow."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .auto_recharge_pref_list_response import AutoRechargePrefListResponse as AutoRechargePrefListResponse
from .auto_recharge_pref_update_params import AutoRechargePrefUpdateParams as AutoRechargePrefUpdateParams
from .auto_recharge_pref_update_response import AutoRechargePrefUpdateResponse as AutoRechargePrefUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncPerPagePaginationV2, AsyncPerPagePaginationV2
from telnyx.types.messaging_10dlc import (
    TelnyxDownstreamCampaign,
    PartnerCampaignListSharedByMeResponse,
    PartnerCampaignRetrieveSharingStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPartnerCampaigns:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.retrieve(
            "campaignId",
        )
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve(
            "campaignId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = response.parse()
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.messaging_10dlc.partner_campaigns.with_streaming_response.retrieve(
            "campaignId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = response.parse()
            assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `campaign_id` but received ''"):
            client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.update(
            campaign_id="campaignId",
        )
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.update(
            campaign_id="campaignId",
            webhook_failover_url="https://webhook.com/9010a453-4df8-4be6-a551-1070892888d6",
            webhook_url="https://webhook.com/67ea78a8-9f32-4d04-b62d-f9502e8e5f93",
        )
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.partner_campaigns.with_raw_response.update(
            campaign_id="campaignId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = response.parse()
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Telnyx) -> None:
        with client.messaging_10dlc.partner_campaigns.with_streaming_response.update(
            campaign_id="campaignId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = response.parse()
            assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `campaign_id` but received ''"):
            client.messaging_10dlc.partner_campaigns.with_raw_response.update(
                campaign_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.list()
        assert_matches_type(SyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.list(
            page=0,
            records_per_page=0,
            sort="assignedPhoneNumbersCount",
        )
        assert_matches_type(SyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.partner_campaigns.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = response.parse()
        assert_matches_type(SyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.messaging_10dlc.partner_campaigns.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = response.parse()
            assert_matches_type(SyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_shared_by_me(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.list_shared_by_me()
        assert_matches_type(
            SyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_shared_by_me_with_all_params(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.list_shared_by_me(
            page=0,
            records_per_page=0,
        )
        assert_matches_type(
            SyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list_shared_by_me(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.partner_campaigns.with_raw_response.list_shared_by_me()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = response.parse()
        assert_matches_type(
            SyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list_shared_by_me(self, client: Telnyx) -> None:
        with client.messaging_10dlc.partner_campaigns.with_streaming_response.list_shared_by_me() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = response.parse()
            assert_matches_type(
                SyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve_sharing_status(self, client: Telnyx) -> None:
        partner_campaign = client.messaging_10dlc.partner_campaigns.retrieve_sharing_status(
            "campaignId",
        )
        assert_matches_type(PartnerCampaignRetrieveSharingStatusResponse, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve_sharing_status(self, client: Telnyx) -> None:
        response = client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve_sharing_status(
            "campaignId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = response.parse()
        assert_matches_type(PartnerCampaignRetrieveSharingStatusResponse, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve_sharing_status(self, client: Telnyx) -> None:
        with client.messaging_10dlc.partner_campaigns.with_streaming_response.retrieve_sharing_status(
            "campaignId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = response.parse()
            assert_matches_type(PartnerCampaignRetrieveSharingStatusResponse, partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve_sharing_status(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `campaign_id` but received ''"):
            client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve_sharing_status(
                "",
            )


class TestAsyncPartnerCampaigns:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.retrieve(
            "campaignId",
        )
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve(
            "campaignId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = await response.parse()
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_10dlc.partner_campaigns.with_streaming_response.retrieve(
            "campaignId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = await response.parse()
            assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `campaign_id` but received ''"):
            await async_client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.update(
            campaign_id="campaignId",
        )
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.update(
            campaign_id="campaignId",
            webhook_failover_url="https://webhook.com/9010a453-4df8-4be6-a551-1070892888d6",
            webhook_url="https://webhook.com/67ea78a8-9f32-4d04-b62d-f9502e8e5f93",
        )
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.partner_campaigns.with_raw_response.update(
            campaign_id="campaignId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = await response.parse()
        assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_10dlc.partner_campaigns.with_streaming_response.update(
            campaign_id="campaignId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = await response.parse()
            assert_matches_type(TelnyxDownstreamCampaign, partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `campaign_id` but received ''"):
            await async_client.messaging_10dlc.partner_campaigns.with_raw_response.update(
                campaign_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.list()
        assert_matches_type(AsyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.list(
            page=0,
            records_per_page=0,
            sort="assignedPhoneNumbersCount",
        )
        assert_matches_type(AsyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.partner_campaigns.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = await response.parse()
        assert_matches_type(AsyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_10dlc.partner_campaigns.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = await response.parse()
            assert_matches_type(AsyncPerPagePaginationV2[TelnyxDownstreamCampaign], partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_shared_by_me(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.list_shared_by_me()
        assert_matches_type(
            AsyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_shared_by_me_with_all_params(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.list_shared_by_me(
            page=0,
            records_per_page=0,
        )
        assert_matches_type(
            AsyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list_shared_by_me(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.partner_campaigns.with_raw_response.list_shared_by_me()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = await response.parse()
        assert_matches_type(
            AsyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
        )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list_shared_by_me(self, async_client: AsyncTelnyx) -> None:
        async with (
            async_client.messaging_10dlc.partner_campaigns.with_streaming_response.list_shared_by_me()
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = await response.parse()
            assert_matches_type(
                AsyncPerPagePaginationV2[PartnerCampaignListSharedByMeResponse], partner_campaign, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve_sharing_status(self, async_client: AsyncTelnyx) -> None:
        partner_campaign = await async_client.messaging_10dlc.partner_campaigns.retrieve_sharing_status(
            "campaignId",
        )
        assert_matches_type(PartnerCampaignRetrieveSharingStatusResponse, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve_sharing_status(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve_sharing_status(
            "campaignId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        partner_campaign = await response.parse()
        assert_matches_type(PartnerCampaignRetrieveSharingStatusResponse, partner_campaign, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve_sharing_status(self, async_client: AsyncTelnyx) -> None:
        async with async_client.messaging_10dlc.partner_campaigns.with_streaming_response.retrieve_sharing_status(
            "campaignId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            partner_campaign = await response.parse()
            assert_matches_type(PartnerCampaignRetrieveSharingStatusResponse, partner_campaign, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve_sharing_status(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `campaign_id` but received ''"):
            await async_client.messaging_10dlc.partner_campaigns.with_raw_response.retrieve_sharing_status(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["InboundChannelUpdateParams"]


class InboundChannelUpdateParams(TypedDict, total=False):
    channels: Required[int]
    """The new number of concurrent channels for the account"""
